"""
gitspy._subprocess
==================
"""
import os as _os
import subprocess as _sp
import typing as _t
from pathlib import Path as _Path

from spall import Subprocess as _Subprocess

from . import exceptions as _exceptions
from ._environ import TempEnvVar as _TempEnvVar


class Git(_Subprocess):
    """Git commands as class attributes.

    @DynamicAttrs
    """

    def __init__(self) -> None:
        self.commands = [
            i.lstrip().split()[0]
            for i in _sp.check_output(["git", "help", "--all"])
            .decode()
            .splitlines()
            if i.startswith("   ")
        ]
        super().__init__("git", positionals=self.commands, loglevel="debug")

    def call(self, *args: str, **kwargs: _t.Union[bool, str]) -> int:
        """Call partial git command instantiated in superclass.

        :param args: Command's positional arguments.
        :key file: File path to write the stdout stream to.
        :key capture: Pipe stream to self.
        :key devnull: Suppress output.
        :key suppress: Suppress errors and continue running.
        :raises NotARepositoryError: If not run from within a
            repository.
        :raises CalledProcessError: If error occurs in subprocess.
        :return: Exit status.
        """
        git_dir = _Path.cwd() / ".git"
        with _TempEnvVar(GIT_WORK_TREE=str(_Path.cwd()), GIT_DIR=str(git_dir)):
            if "--bare" in args:
                del _os.environ["GIT_WORK_TREE"]

            try:
                return super().call(*args, **kwargs)

            except _sp.CalledProcessError as err:
                if not git_dir.is_dir():
                    raise _exceptions.NotARepositoryError from err

                raise err



"""
gitspy.exceptions
=================

Exceptions for use within the module.

All exceptions made public for if they need to be reraised or excepted.

Exceptions are already built into the architecture but can be used in
new plugins as well.
"""


class NotARepositoryError(OSError):
    """Raise if there is an error related to a Git repository."""

    def __init__(self) -> None:
        super().__init__("not a git repository")



"""
gitspy.config
=============
"""
from __future__ import annotations

import os as _os
import typing as _t


class TempEnvVar:
    """Temporarily set an environment variable for context's duration.

    Ensures the environment maintains its state.

    If key already existed, once done, change the key back to its
    original value. If key did not already exist, then delete it.

    :param kwargs: Arbitrary number of key-value pairs to alter.
    """

    def __init__(self, **kwargs: str) -> None:
        self._default = {k: _os.environ.get(k) for k in kwargs}
        _os.environ.update(kwargs)

    def __enter__(self) -> TempEnvVar:
        return self

    def __exit__(
        self, exc_type: _t.Any, exc_val: _t.Any, exc_tb: _t.Any
    ) -> None:
        for key, value in self._default.items():
            if value is None:
                try:
                    del _os.environ[key]
                except KeyError:
                    # in the case that key gets deleted within context
                    pass
            else:
                _os.environ[key] = str(self._default[key])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gitspy']

package_data = \
{'': ['*']}

install_requires = \
['spall>=0.1.0,<0.2.0']

setup_kwargs = {
    'name': 'gitspy',
    'version': '0.1.1',
    'description': 'Intuitive Git for Python',
    'long_description': 'gitspy\n======\n.. image:: https://github.com/jshwi/gitspy/actions/workflows/ci.yml/badge.svg\n    :target: https://github.com/jshwi/gitspy/actions/workflows/ci.yml\n    :alt: ci\n.. image:: https://img.shields.io/badge/python-3.8-blue.svg\n    :target: https://www.python.org/downloads/release/python-380\n    :alt: python3.8\n.. image:: https://img.shields.io/pypi/v/gitspy\n    :target: https://img.shields.io/pypi/v/gitspy\n    :alt: pypi\n.. image:: https://codecov.io/gh/jshwi/gitspy/branch/master/graph/badge.svg\n    :target: https://codecov.io/gh/jshwi/gitspy\n    :alt: codecov.io\n.. image:: https://img.shields.io/badge/License-MIT-blue.svg\n    :target: https://lbesson.mit-license.org/\n    :alt: mit\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n    :target: https://github.com/psf/black\n    :alt: black\n\nIntuitive Git for Python\n\n\nInstall\n-------\nDependencies: git ^2.0.0 (tested)\n\n``pip install gitspy``\n\nDevelopment\n\n``poetry install``\n\nExample Usage\n-------------\n\nGet branch\n\n.. code-block:: python\n\n    >>> import gitspy\n    >>> git = gitspy.Git()\n    >>> # capture will store stdout, which can then be consumed by\n    >>> # calling `git.stdout()`\n    >>> # default is to print stdout, and stderr, to console\n    >>> returncode = git.symbolic_ref("--short", "HEAD", capture=True)\n    >>> print(returncode)  # printing returncode\n    0\n    >>> # consume stdout, a list containing a `str` of the checked out\n    >>> # branch\n    >>> stdout = git.stdout()  # -> [\'checked-out-branch\']\n    >>> items = len(stdout)  # printing length of `stdout()` outputs\n    >>> print(items)\n    1\n    >>> # no commands have been called yet since last call to `stdout`,\n    >>> # so stdout is empty\n    >>> stdout = git.stdout()  # -> []\n    >>> items = len(stdout)  # printing length of ``stdout()`` outputs\n    >>> print(items)\n    0\n    >>> # stdout can be accrued\n    >>> # [\'checked-out-branch\', \'checked-out-branch\']\n    >>> git.symbolic_ref("--short", "HEAD", capture=True)\n    >>> git.symbolic_ref("--short", "HEAD", capture=True)\n    >>> print(len(git.stdout()))\n    2\n    >>> # stdout is consumed\n    >>> print(len(git.stdout()))\n    0\n    >>> git.symbolic_ref("--short", "HEAD", capture=True)\n    >>> git.stdout()  # [...] -> void; clear stdout, if it exists\n    >>> print(len(git.stdout()))\n    0\n..\n\nGet commit hash\n\n.. code-block:: python\n\n    >>> import gitspy\n    >>> git = gitspy.Git()\n    >>> git.rev_parse("HEAD", capture=True)\n    >>> stored = git.stdout()[0]\n    >>> print(len(stored))  # print the length of the unique hash\n    40\n..\n',
    'author': 'jshwi',
    'author_email': 'stephen@jshwisolutions.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
